from slapos.cloudmgr.lib import getDriverInstance, getNode
import sys
def start(key, secret, service, node_uuid):
  """Starts node"""
  driver = getDriverInstance(service, key, secret)
  node = getNode(driver, node_uuid)
  if node is None:
    return False
  if node.state in [0, 1]:
    return True
  elif node.state == 3:
    return node.reboot()
  else:
    return False
  return True

def main():
  print start(sys.argv[1], sys.argv[2], sys.argv[3], sys.argv[4])



# pylint: disable-msg=W0311,C0103

import os
import signal
import time
import xmlrpclib

# This mini-library is used to communicate with supervisord process
# It aims to replace the file "process.py"
# For the moment, we keep both for compatibility


def isRunning(config, process):
  """
  Ask supervisor if given process is currently running
  """
  server = xmlrpclib.Server(config['supervisord_server'])
  state = server.supervisor.getProcessInfo(process)['state']
  return (True if state in (10, 20) else False)


def killRunningProcess(config, process, sig=signal.SIGTERM):
  """
  Send signal "sig" to given process.
  Default signal sent is SIGTERM
  """
  server = xmlrpclib.Server(config['supervisord_server'])
  pid = server.supervisor.getProcessInfo(process)['pid']
  if pid != 0:
    os.kill(pid, sig)


def returnCode(config, process):
  """
  Get the returned code of the last run of given process
  """
  server = xmlrpclib.Server(config['supervisord_server'])
  code = server.supervisor.getProcessInfo(process)['exitstatus']
  return code


def runProcess(config, process):
  """
  Start a process registered by supervisor
  """
  server = xmlrpclib.Server(config['supervisord_server'])
  server.supervisor.startProcess(process)


def runProcesses(config, processes):
  """
  Start by supervisor a list of given processes, one by one
  """
  server = xmlrpclib.Server(config['supervisord_server'])
  for proc in processes:
    server.supervisor.startProcess(proc)
    waitForProcessEnd(config, proc)


def stopProcess(config, process):
  """
  Ask supervisor to stop a process
  """
  if isRunning(config, process):
    server = xmlrpclib.Server(config['supervisord_server'])
    server.supervisor.stopProcess(process)


def stopProcesses(config, processes):
  """
  Stop a list of processes
  """
  server = xmlrpclib.Server(config['supervisord_server'])
  for proc in processes:
    server.supervisor.stopProcess(proc)


def waitForProcessEnd(config, process):
  """
  Block program's execution until given process quits Running state
  """
  server = xmlrpclib.Server(config['supervisord_server'])
  while True:
    state = server.supervisor.getProcessInfo(process)['state']
    if state == 20:
      time.sleep(3)
    else:
      return True
  return False



# -*- coding: utf-8 -*-

import ConfigParser
import argparse
import gdbm

import sys
import os
import logging

from . import process

def main():
    parser = argparse.ArgumentParser(description="Slapcontainer binary")
    parser.add_argument('configuration_file', type=str,
                        help="SlapOS configuration file.")
    parser.add_argument('database', type=str,
                        help='slapcontainer database')
    log_lvls = [lvl for lvl in logging._levelNames.keys()
                if isinstance(lvl, basestring)]
    parser.add_argument('--log', nargs=1, default=['INFO'],
                        choices=log_lvls,
                        metavar='lvl', help='Log level')
    parser.add_argument('--pid', nargs=1, help='pid file path')
    args = parser.parse_args()

    if args.pid is not None:
        pid_filename = args.pid[0]
        if os.path.exists(pid_filename):
            print >> sys.stderr, "Already running"
            return 127
        with open(pid_filename, 'w') as pid_file:
            pid_file.write(str(os.getpid()))

        try:
            run(args)
        finally:
            os.remove(pid_filename)

    else:
        run(args)

def run(args):
    slapos_conf = ConfigParser.ConfigParser()
    slapos_conf.read(args.configuration_file)

    current_binary = os.path.join(os.getcwd(), sys.argv[0])
    binary_directory = os.path.dirname(current_binary)
    sr_directory = os.path.realpath(os.path.join(binary_directory, '..'))

    partition_amount = slapos_conf.getint('slapformat', 'partition_amount')
    partition_base_name = slapos_conf.get('slapformat', 'partition_base_name')
    try:
        bridge_name = slapos_conf.get('slapformat', 'interface_name')
    except ConfigParser.NoOptionError:
        bridge_name = slapos_conf.get('slapformat', 'bridge_name')
    instance_root = slapos_conf.get('slapos', 'instance_root')
    partition_base_path = os.path.join(instance_root, partition_base_name)
    partition_list = ['%s%d' % (partition_base_path, i)
                      for i in range(partition_amount)]

    logging.basicConfig(level=logging.getLevelName(args.log[0]))

    database = gdbm.open(args.database, 'c', 0600)
    try:
        process.main(sr_directory, partition_list, database, bridge_name)
    finally:
        database.sync()
        database.close()



from slapos.cloudmgr.lib import getDriverInstance, getNode
from libcloud.types import NodeState
import sys
def destroy(key, secret, service, node_uuid):
  driver = getDriverInstance(service, key, secret)
  node = getNode(driver, node_uuid)
  if node is None:
    return False
  if node.state in [NodeState.RUNNING, NodeState.REBOOTING, NodeState.PENDING]:
    return node.destroy()
  else:
    return False

def main():
  print destroy(sys.argv[1], sys.argv[2], sys.argv[3], sys.argv[4])



# -*- coding: utf-8 -*-
# vim: set et sts=2:
# pylint: disable-msg=W0311,C0301,C0103,C0111


import os
import re
import shutil

from git import Repo
from flask import jsonify


def cloneRepo(data):
  """Clone a repository
  Args:
    data: a dictionary of parameters to use:
      data['path'] is the path of the new project
      data['repo'] is the url of the repository to be cloned
      data['email'] is the user's email
      data['user'] is the name of the user
  Returns:
    a jsonify data"""
  workDir = data['path']
  if not workDir:
    return jsonify(code=0,
                   result="Can not create project folder.")
  code = 0
  json = ""
  try:
    if os.path.exists(workDir) and len(os.listdir(workDir)) < 2:
      shutil.rmtree(workDir)  # delete useless files
    repo = Repo.clone_from(data["repo"], workDir)
    config_writer = repo.config_writer()
    config_writer.add_section("user")
    if data["user"] != "":
      config_writer.set_value("user", "name", data["user"].encode("utf-8"))
    if data["email"] != "":
      config_writer.set_value("user", "email", data["email"])
    code = 1
  except Exception as e:
    json = safeResult(str(e))
  return jsonify(code=code, result=json)


def gitStatus(project):
  """Run git status and return status of specified project folder
  Args:
    project: path of the projet to get status
  Returns:
    a parsed string that contains the result of git status"""
  code = 0
  json = ""
  try:
    repo = Repo(project)
    git = repo.git
    json = git.status().replace('#', '')
    branch = git.branch().replace(' ', '').split('\n')
    isdirty = repo.is_dirty(untracked_files=True)
    code = 1
  except Exception as e:
    json = safeResult(str(e))
  return jsonify(code=code, result=json, branch=branch, dirty=isdirty)


def switchBranch(project, name):
  """Switch a git branch
  Args:
    project: directory of the local git repository
    name: switch from current branch to `name` branch
  Returns:
    a jsonify data"""
  code = 0
  json = ""
  try:
    repo = Repo(project)
    current_branch = repo.active_branch.name
    if name == current_branch:
      json = "This is already your active branch for this project"
    else:
      git = repo.git
      git.checkout(name)
      code = 1
  except Exception as e:
    json = safeResult(str(e))
  return jsonify(code=code, result=json)


def addBranch(project, name, onlyCheckout=False):
  """Add new git branch to the repository
  Args:
    project: directory of the local git repository
    name: name of the new branch
    onlyCheckout: if True then the branch `name` is created before checkout
  Returns:
    a jsonify data"""
  code = 0
  json = ""
  try:
    repo = Repo(project)
    git = repo.git
    if not onlyCheckout:
      git.checkout('-b', name)
    else:
      git.checkout(name)
    code = 1
  except Exception as e:
    json = safeResult(str(e))
  return jsonify(code=code, result=json)


def getDiff(project):
  """Get git diff for the specified project directory"""
  result = ""
  try:
    repo = Repo(project)
    git = repo.git
    current_branch = repo.active_branch.name
    result = git.diff(current_branch)
  except Exception as e:
    result = safeResult(str(e))
  return result

def gitCommit(project, msg):
  """Commit changes for the specified repository
  Args:
    project: directory of the local repository
    msg: commit message"""
  code = 0
  json = ""
  repo = Repo(project)
  if repo.is_dirty:
    git = repo.git
    #add file to be commited
    files = repo.untracked_files
    for f in files:
      git.add(f)
    #Commit all modified and untracked files
    git.commit('-a', '-m', msg)
    code = 1
  else:
    json = "Nothing to be commited"
  return jsonify(code=code, result=json)

def gitPush(project):
  """Push changes for the specified repository
  Args:
    project: directory of the local repository
    msg: commit message"""
  code = 0
  json = ""
  try:
    repo = Repo(project)
    git = repo.git
    #push changes to repo
    current_branch = repo.active_branch.name
    git.push('origin', current_branch)
    code = 1
  except Exception as e:
    json = safeResult(str(e))
  return jsonify(code=code, result=json)


def gitPull(project):
  result = ""
  code = 0
  try:
    repo = Repo(project)
    git = repo.git
    git.pull()
    code = 1
  except Exception as e:
    result = safeResult(str(e))
  return jsonify(code=code, result=result)


def safeResult(result):
  """Parse string and remove credential of the user"""
  regex = re.compile("(https:\/\/)([\w\d\._-]+:[\w\d\._-]+)\@([\S]+\s)", re.VERBOSE)
  return regex.sub(r'\1\3', result)
